@@ -1,6 +1,9 @@
+from functools import partial
 from re import compile as rc
 from datetime import datetime
 from io import BytesIO, StringIO
+from ast import literal_eval
+
 
 from requests import Session
 from jdatetime import datetime as jdatetime
@@ -28,7 +31,7 @@ MONTH_AVG_VOL_SEARCH = rc(r"QTotTran5JAvg='(\d+)'").search
 FIRST_NUMBER_SEARCH = rc(r'\d+').search
 INSTANT_MATCH = rc(
     r'(?P<timestamp>\d\d:\d\d:\d\d),'
-    r'..,'
+    r'[^,]+,'
     r'(?P<pl>\d+),'
     r'(?P<pc>\d+),'
     r'(?P<pf>\d+),'
@@ -38,12 +41,15 @@ INSTANT_MATCH = rc(
     r'(?P<tno>\d+),'
     r'(?P<tvol>\d+),'
     r'(?P<tval>\d+),'
-    r'1,'
+    r'\d+,'
     r'(?P<last_info_datetime>\d+,\d+)'
     r'(,(?P<nav_datetime>[\d\/: ]+),(?P<nav>\d+))?'
 ).match
 INSTANT_INTS = {
     'pl', 'pc', 'py', 'pf', 'py', 'pmin', 'pmax', 'tno', 'tvol', 'tval'}
+RELATED_COMPANIES = rc(r"var RelatedCompanies=(\[.*\]);").search
+TRADE_HISTORY = rc(r"var TradeHistory=(\[.*\]);").search
+STR_TO_NUM = partial(rc(r"'([\d.]+)'").sub, r'\1')
 
 
 def get_content(url) -> bytes:
@@ -63,7 +69,7 @@ class Instrument:
         self.id = id
 
     def get_page_info(self) -> dict:
-        """Return the static info of the symbol's page.
+        """Return the static info found on instrument's page.
 
         For the meaning of keys see:
             https://cdn.tsetmc.com/Site.aspx?ParTree=151713
@@ -75,6 +81,9 @@ class Instrument:
         free_float_match = FREE_FLOAT_SEARCH(text)
         eps_match = EPS_SEARCH(text)
         sector_pe_match = SECTOR_PE_SEARCH(text)
+        trade_history = literal_eval(STR_TO_NUM(TRADE_HISTORY(text)[1]))
+        trade_history = DataFrame(trade_history, columns=('date', 'pc', 'py', 'pmin', 'pmax', 'tno', 'tvol', 'tval'))
+        trade_history.set_index('date', inplace=True)
         return {
             'tmax': float(t_min_max[2])
             , 'tmin': float(t_min_max[1])
@@ -94,6 +103,8 @@ class Instrument:
             , 'week_min': float(wy_min_max[1])
             , 'year_max': float(wy_min_max[4])
             , 'year_min': float(wy_min_max[3])
+            , 'related_companies': literal_eval(STR_TO_NUM(RELATED_COMPANIES(text)[1]))
+            , 'trade_history': trade_history
         }
 
     def get_info(self) -> dict:
@@ -152,22 +163,22 @@ def get_market_watch_init() -> dict:
     text = fa_norm_text('http://tsetmc.com/tsev2/data/MarketWatchInit.aspx?h=0&r=0')
     _, _, states, price_rows, _ = text.split('@')
     state_df = read_csv(
-        StringIO(states),
-        lineterminator=';',
-        names=(
+        StringIO(states)
+        , lineterminator=';'
+        , names=(
             'id', 'isin', 'l18', 'l30', 'heven', 'pf', 'pc', 'pl', 'tno'
             , 'tvol', 'tval', 'pmin', 'pmax', 'py', 'eps', 'bvol', 'visitcount'
-            , 'flow', 'cs', 'tmax', 'tmin', 'z', 'yval'),
+            , 'flow', 'cs', 'tmax', 'tmin', 'z', 'yval')
         # unlike int64, Int64 is nullable
-        dtype={'tmin': "Int64", 'tmax': "Int64"},
-        low_memory=False,
-        index_col=['id', 'isin', 'l18', 'l30'])
+        , dtype={'tmin': "Int64", 'tmax': "Int64"}
+        , low_memory=False
+        , index_col=['id', 'isin', 'l18', 'l30'])
     price_df = read_csv(
-        StringIO(price_rows),
-        lineterminator=';',
-        names=('id', 'row', 'zo', 'zd', 'pd', 'po', 'qd', 'qo'),
-        dtype="Int64",
-        low_memory=False)
+        StringIO(price_rows)
+        , lineterminator=';'
+        , names=('id', 'row', 'zo', 'zd', 'pd', 'po', 'qd', 'qo')
+        , dtype="Int64"
+        , low_memory=False)
     # merge multiple rows sharing the same `row` number into one row.
     # a fascinating solution from https://stackoverflow.com/a/53563551/2705757
     price_df.set_index(['id', 'row'], inplace=True)
@@ -199,6 +210,9 @@ def get_closing_price_all() -> DataFrame:
 
     For the meaning of column names refer to
         http://cdn.tsetmc.com/Site.aspx?ParTree=151713
+    This method returns the same set of information as `[ih]` variable in
+        tsetmc filters. See:
+            http://tsetmc.com/Site.aspx?ParTree=151715&LnkIdn=3197
     """
     content = get_content('http://members.tsetmc.com/tsev2/data/ClosingPriceAll.aspx')
     data = _split_id_rows(content, id_row_len=11)
